import 'dart:ui';

import 'package:flutter/widgets.dart';
import 'package:states_rebuilder/states_rebuilder.dart';

final currentLocale = RM.inject<Locale>(
  () {
    final systemLocale = WidgetsBinding.instance.window.locales.first;
    if (I18n.supportedLocal.keys.contains(systemLocale)) {
      return systemLocale;
    }
    return I18n.supportedLocal.keys.first;
  },
);
final i18n = RM.inject<I18n>(
  () => I18n.supportedLocal[currentLocale.state],
  dependsOn: DependsOn({currentLocale}),
);

abstract class I18n {
  static final supportedLocal = {
    Locale('en'): EN(),
    Locale('fr'): ES(),
    Locale('ar'): AR(),
    Locale('de'): DE(),
  };
  final String languageName;
  I18n(this.languageName);
  final helloWorldExample = 'Hello world Example';
  final startStreaming = 'Start Streaming';
  final enterYourName = 'Enter your name';
}

class EN extends I18n {
  EN() : super('English');
}

class ES implements I18n {
  @override
  String get languageName => 'Spanish';

  final helloWorldExample = 'Hola mundo Ejemplo';
  final startStreaming = 'Comenzar a transmitir';
  final enterYourName = 'Introduzca su nombre';
}

class AR implements I18n {
  @override
  String get languageName => 'Arabic';

  final helloWorldExample = 'مثال مرحبا بالعالم';
  final startStreaming = 'إبدا التدفق';
  final enterYourName = 'إدخل إسمك';
}

class DE implements I18n {
  @override
  String get languageName => 'German';

  final helloWorldExample = 'Hallo Welt Beispiel';
  final startStreaming = 'Starten Sie das Streaming';
  final enterYourName = 'Gib deinen Namen ein';
}
